#ifndef __MEM_H__
#define __MEM_H__

/**
 * mem.h
 *   Some useful functions for exploring memory usage.
 */

// +-----------+------------------------------------------------------
// | Functions |
// +-----------+

/**
 * Procedure:
 *   mem_allocated
 * Parameters:
 *   [None]
 * Purpose:
 *   Determine how much memory has been allocated by malloc and not 
 *   subsequently freed.
 * Produces:
 *   ma, an integer.
 * Preconditions:
 *   [No additional.]
 * Postconditions:
 *   ma is the number of bytes that malloc has allocated.  (At least as
 *   reported by the various malloc utilities.)
 */
int mem_allocated (void);


#endif // __MEM___
