#include "mem.h"
#include <stdio.h>
#include <stdlib.h>

int 
main ()
{
  int i;
  char *stuff;
  for (i = 0; i < 64; i++)
    {
      stuff = (char *) malloc (i);
      printf ("For %d, %d bytes have been allocated\n", i, mem_allocated());
      free (stuff);
      printf ("Back down to %d\n", mem_allocated ());
    }
  return 0;
}
